#! /usr/bin/env python

import rospy
import os
import sys
import subprocess
from std_srvs.srv import Empty, EmptyResponse


class DummyProcess:
    def __init__(self):
        pass

    def terminate(self):
        pass


class RobustGazeboLaunch:
    def __init__(self, launch_cmd):
        rospy.init_node('robust_gazebo_launch')
        self.launch_cmd = launch_cmd
        self.gazebo_process = DummyProcess()

        rospy.Service('/start_gazebo', Empty, self.start_service_cb)
        rospy.Service('/stop_gazebo', Empty, self.stop_service_cb)

        auto_start = rospy.get_param('~auto_start', default=True)
        if auto_start:
            self.start_gazebo()

    def start_service_cb(self, request):
        self.start_gazebo()
        return EmptyResponse()

    def stop_service_cb(self, request):
        self.stop_gazebo()
        return EmptyResponse()

    def start_gazebo(self):
        self.stop_gazebo()
        self.gazebo_process = subprocess.Popen(self.launch_cmd.split())

    def stop_gazebo(self):
        self.gazebo_process.terminate()
        os.system('killall gzserver')
        os.system('killall gzclient')

    def shutdown_handler(self):
        self.stop_gazebo()


if __name__ == '__main__':
    launch_cmd = 'roslaunch'
    for a in sys.argv:
        if '.py' not in a and '__name' not in a and '__log' not in a:
            launch_cmd += (' ' + a)

    node_instance = RobustGazeboLaunch(launch_cmd)
    rospy.on_shutdown(node_instance.shutdown_handler)
    rospy.spin()
